@@ -1,28 +1,41 @@
-# -*- coding: utf-8 -*-
-"""Base class for Repository objects.
-
-libvcs.base
-~~~~~~~~~~~
-
-"""
-from __future__ import absolute_import, print_function, unicode_literals
-
+"""Base class for Repository objects."""
 import logging
 import os
+from typing import NamedTuple
+from urllib import parse as urlparse
 
-from ._compat import implements_to_string, urlparse
 from .util import RepoLoggingAdapter, mkdir_p, run
 
 logger = logging.getLogger(__name__)
 
 
-@implements_to_string
+class VCSLocation(NamedTuple):
+    url: str
+    rev: str
+
+
+def convert_pip_url(pip_url: str) -> VCSLocation:
+    """Return repo URL and revision by parsing `libvcs.base.BaseRepo.url`."""
+    error_message = (
+        "Sorry, '%s' is a malformed VCS url. "
+        "The format is <vcs>+<protocol>://<url>, "
+        "e.g. svn+http://myrepo/svn/MyApp#egg=MyApp"
+    )
+    assert '+' in pip_url, error_message % pip_url
+    url = pip_url.split('+', 1)[1]
+    scheme, netloc, path, query, frag = urlparse.urlsplit(url)
+    rev = None
+    if '@' in path:
+        path, rev = path.rsplit('@', 1)
+    url = urlparse.urlunsplit((scheme, netloc, path, query, ''))
+    return VCSLocation(url=url, rev=rev)
+
+
 class BaseRepo(RepoLoggingAdapter, object):
 
     """Base class for repositories.
 
     Extends :py:class:`logging.LoggerAdapter`.
-
     """
 
     #: log command output to buffer
@@ -33,20 +46,24 @@ class BaseRepo(RepoLoggingAdapter, object):
 
     def __init__(self, url, repo_dir, progress_callback=None, *args, **kwargs):
         """
-        :param callback: Retrieve live progress from ``sys.stderr`` (useful for
-            certain vcs commands like ``git pull``. Use ``progress_callback``::
+        Parameters
+        ----------
+        callback : func
+            Retrieve live progress from ``sys.stderr`` (useful for certain vcs commands
+            like ``git pull``. Use ``progress_callback``:
 
-                def progress_cb(output, timestamp):
-                    sys.stdout.write(output)
-                    sys.stdout.flush()
-                create_repo(..., progress_callback=progress_cb)
-        :type callback: func
+            >>> def progress_cb(output, timestamp):
+            >>>     sys.stdout.write(output)
+            >>>     sys.stdout.flush()
+            >>> create_repo(..., progress_callback=progress_cb)
         """
         self.progress_callback = progress_callback
         self.url = url
         self.parent_dir = os.path.dirname(repo_dir)
         self.repo_name = os.path.basename(os.path.normpath(repo_dir))
         self.path = repo_dir
+        if 'rev' in kwargs:
+            self.rev = kwargs['rev']
 
         # Register more schemes with urlparse for various version control
         # systems
@@ -60,7 +77,7 @@ class BaseRepo(RepoLoggingAdapter, object):
 
     @classmethod
     def from_pip_url(cls, pip_url, *args, **kwargs):
-        url, rev = cls.get_url_and_revision_from_pip_url(pip_url)
+        url, rev = convert_pip_url(pip_url)
         self = cls(url=url, rev=rev, *args, **kwargs)
 
         return self
@@ -77,17 +94,21 @@ class BaseRepo(RepoLoggingAdapter, object):
         """Return combined stderr/stdout from a command.
 
         This method will also prefix the VCS command bin_name. By default runs
-        using the cwd :attr:`~.path` of the repo.
+        using the cwd `libvcs.base.BaseRepo.path` of the repo.
 
-        :param cwd: dir command is run from, defaults :attr:`~.path`.
-        :type cwd: str
+        Parameters
+        ----------
+        cwd : str
+            dir command is run from, defaults to `libvcs.base.BaseRepo.path`.
 
-        :param check_returncode: Indicate whether a :exc:`~exc.CommandError`
-            should be raised if return code is different from 0.
-        :type check_returncode: :class:`bool`
+        check_returncode : bool
+            Indicate whether a :exc:`~exc.CommandError` should be raised if return code
+            is different from 0.
 
-        :returns: combined stdout/stderr in a big string, newlines retained
-        :rtype: str
+        Returns
+        -------
+        str
+            combined stdout/stderr in a big string, newlines retained
         """
 
         if cwd is None:
@@ -105,7 +126,7 @@ class BaseRepo(RepoLoggingAdapter, object):
             cwd=cwd,
         )
 
-    def check_destination(self, *args, **kwargs):
+    def ensure_dir(self, *args, **kwargs):
         """Assure destination path exists. If not, create directories."""
         if os.path.exists(self.path):
             return True
@@ -122,22 +143,5 @@ class BaseRepo(RepoLoggingAdapter, object):
 
         return True
 
-    @classmethod
-    def get_url_and_revision_from_pip_url(cls, pip_url):
-        """Return repo URL and revision by parsing :attr:`~.url`."""
-        error_message = (
-            "Sorry, '%s' is a malformed VCS url. "
-            "The format is <vcs>+<protocol>://<url>, "
-            "e.g. svn+http://myrepo/svn/MyApp#egg=MyApp"
-        )
-        assert '+' in pip_url, error_message % pip_url
-        url = pip_url.split('+', 1)[1]
-        scheme, netloc, path, query, frag = urlparse.urlsplit(url)
-        rev = None
-        if '@' in path:
-            path, rev = path.rsplit('@', 1)
-        url = urlparse.urlunsplit((scheme, netloc, path, query, ''))
-        return url, rev
-
     def __repr__(self):
         return "<{} {}>".format(self.__class__.__name__, self.repo_name)
