@@ -1,52 +1,57 @@
-# -*- coding: utf-8 -*-
 """Git Repo object for libvcs.
 
-libvcs.git
-~~~~~~~~~~
+.. todo::
 
-From https://github.com/saltstack/salt (Apache License):
+    From https://github.com/saltstack/salt (Apache License):
 
-- :py:meth:`GitRepo.remote`
-- :py:meth:`GitRepo.remote_get` (renamed to ``remote``)
-- :py:meth:`GitRepo.remote_set` (renamed to ``set_remote``)
+    - [`GitRepo.remote`](libvcs.git.GitRepo.remote) (renamed to ``remote``)
+    - [`GitRepo.remote`](libvcs.git.GitRepo.remote_set) (renamed to ``set_remote``)
 
-From pip (MIT Licnese):
+    From pip (MIT Licnese):
 
-- :py:meth:`GitRepo.get_url_and_revision_from_pip_url` (get_url_rev)
-- :py:meth:`GitRepo.get_revision`
-- :py:meth:`GitRepo.get_git_version`
-
-"""
-from __future__ import absolute_import, print_function, unicode_literals
-
-import collections
+    - [`GitRepo.remote`](libvcs.git.GitRepo.remote_set) (renamed to ``set_remote``)
+    - [`GitRepo.convert_pip_url`](libvcs.git.GitRepo.convert_pip_url`) (``get_url_rev``)
+    - [`GitRepo.get_revision`](libvcs.git.GitRepo.get_revision)
+    - [`GitRepo.get_git_version`](libvcs.git.GitRepo.get_git_version)
+"""  # NOQA: E501
 import logging
 import os
 import re
-import warnings
+from typing import Dict, NamedTuple, Optional
+from urllib import parse as urlparse
 
 from . import exc
-from ._compat import urlparse
-from .base import BaseRepo
+from .base import BaseRepo, VCSLocation, convert_pip_url as base_convert_pip_url
 
 logger = logging.getLogger(__name__)
 
-GitRemote = collections.namedtuple('GitRemote', ['name', 'fetch_url', 'push_url'])
-"""Structure containing git repo information.
 
-Supports :meth:`collections.namedtuple._asdict()`
+class GitRemote(NamedTuple):
+    """Structure containing git repo information.
 
-.. versionadded:: 0.4.0
-"""
+    Supports `collections.namedtuple._asdict()`
+    """
+
+    name: str
+    fetch_url: str
+    push_url: str
 
 
-def extract_status(value):
+class GitStatus(NamedTuple):
+    branch_oid: Optional[str]
+    branch_head: Optional[str]
+    branch_upstream: Optional[str]
+    branch_ab: Optional[str]
+    branch_ahead: Optional[str]
+    branch_behind: Optional[str]
+
+
+def extract_status(value) -> GitStatus:
     """Returns ``git status -sb --porcelain=2`` extracted to a dict
 
     Returns
     -------
-    dict : 
-        Dictionary of git repo's status
+    Dictionary of git repo's status
     """
     pattern = re.compile(
         r"""[\n\r]?
@@ -66,7 +71,7 @@ def extract_status(value):
             (?P<branch_head>
                 .*
             )
-            
+
         )?
         (
             #
@@ -92,49 +97,62 @@ def extract_status(value):
         re.VERBOSE | re.MULTILINE,
     )
     matches = pattern.search(value)
-    return matches.groupdict()
+    return GitStatus(**matches.groupdict())
+
+
+def convert_pip_url(pip_url: str) -> VCSLocation:
+    """
+    Prefixes stub URLs like 'user@hostname:user/repo.git' with 'ssh://'.
+    That's required because although they use SSH they sometimes doesn't
+    work with a ssh:// scheme (e.g. Github). But we need a scheme for
+    parsing. Hence we remove it again afterwards and return it as a stub.
+    The manpage for git-clone(1) refers to this as the "scp-like styntax".
+    """
+    if '://' not in pip_url:
+        assert 'file:' not in pip_url
+        pip_url = pip_url.replace('git+', 'git+ssh://')
+        url, rev = base_convert_pip_url(pip_url)
+        url = url.replace('ssh://', '')
+    elif 'github.com:' in pip_url:
+        raise exc.LibVCSException(
+            "Repo %s is malformatted, please use the convention %s for"
+            "ssh / private GitHub repositories."
+            % (pip_url, "git+https://github.com/username/repo.git")
+        )
+    else:
+        url, rev = base_convert_pip_url(pip_url)
+
+    return VCSLocation(url=url, rev=rev)
 
 
 class GitRepo(BaseRepo):
     bin_name = 'git'
     schemes = ('git', 'git+http', 'git+https', 'git+ssh', 'git+git', 'git+file')
 
-    def __init__(self, url, **kwargs):
+    def __init__(self, url, repo_dir, **kwargs):
         """A git repository.
 
-        :param url: URL of repo
-        :type url: str
+        Parameters
+        ----------
+        url : str
+            URL of repo
 
-        :param git_shallow: clone with ``--depth 1`` (default False)
-        :type git_shallow: bool
-
-        :param git_submodules: Git submodules that shall be updated, all if empty
-        :type git_submodules: list
-
-        :param tls_verify: Should certificate for https be checked (default False)
-        :type tls_verify: bool
-
-        .. versionchanged:: 0.4.0
-
-           The ``remotes`` argument is ignored. Use :meth:`~.set_remote` to set remotes
-           before running :meth:`~.obtain`.
-
-           The ``remotes`` argument is deprecated and will be removed in 0.5
+        tls_verify : bool
+            Should certificate for https be checked (default False)
         """
         if 'git_shallow' not in kwargs:
             self.git_shallow = False
-        if 'git_submodules' not in kwargs:
-            self.git_submodules = []
         if 'tls_verify' not in kwargs:
             self.tls_verify = False
 
-        if kwargs.get('remotes') is not None:
-            warnings.warn(
-                "'remotes' is deprecated and will be removed in 0.5.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-        BaseRepo.__init__(self, url, **kwargs)
+        BaseRepo.__init__(self, url, repo_dir, **kwargs)
+
+    @classmethod
+    def from_pip_url(cls, pip_url, *args, **kwargs):
+        url, rev = convert_pip_url(pip_url)
+        self = cls(url=url, rev=rev, *args, **kwargs)
+
+        return self
 
     def get_revision(self):
         """Return current revision. Initial repositories return 'initial'."""
@@ -143,40 +161,9 @@ class GitRepo(BaseRepo):
         except exc.CommandError:
             return 'initial'
 
-    @classmethod
-    def get_url_and_revision_from_pip_url(cls, pip_url):
-        """
-        Prefixes stub URLs like 'user@hostname:user/repo.git' with 'ssh://'.
-        That's required because although they use SSH they sometimes doesn't
-        work with a ssh:// scheme (e.g. Github). But we need a scheme for
-        parsing. Hence we remove it again afterwards and return it as a stub.
-        The manpage for git-clone(1) refers to this as the "scp-like styntax".
-        """
-        if '://' not in pip_url:
-            assert 'file:' not in pip_url
-            pip_url = pip_url.replace('git+', 'git+ssh://')
-            url, rev = super(GitRepo, cls).get_url_and_revision_from_pip_url(pip_url)
-            url = url.replace('ssh://', '')
-        elif 'github.com:' in pip_url:
-            raise exc.LibVCSException(
-                "Repo %s is malformatted, please use the convention %s for"
-                "ssh / private GitHub repositories."
-                % (pip_url, "git+https://github.com/username/repo.git")
-            )
-        else:
-            url, rev = super(GitRepo, cls).get_url_and_revision_from_pip_url(pip_url)
-
-        return url, rev
-
     def obtain(self):
-        """Retrieve the repository, clone if doesn't exist.
-
-        .. versionchanged:: 0.4.0
-
-           No longer sets remotes. This is now done manually through 
-           :meth:`~.set_remote`.
-        """
-        self.check_destination()
+        """Retrieve the repository, clone if doesn't exist."""
+        self.ensure_dir()
 
         url = self.url
 
@@ -193,11 +180,10 @@ class GitRepo(BaseRepo):
         self.info('Initializing submodules.')
         self.run(['submodule', 'init'], log_in_real_time=True)
         cmd = ['submodule', 'update', '--recursive', '--init']
-        cmd.extend(self.git_submodules)
         self.run(cmd, log_in_real_time=True)
 
     def update_repo(self):
-        self.check_destination()
+        self.ensure_dir()
 
         if not os.path.isdir(os.path.join(self.path, '.git')):
             self.obtain()
@@ -298,7 +284,14 @@ class GitRepo(BaseRepo):
                 except exc.CommandError:
                     self.error("Failed to stash changes")
 
-            # Pull changes from the remote branch
+            # Checkout the remote branch
+            try:
+                process = self.run(['checkout', git_tag])
+            except exc.CommandError:
+                self.error("Failed to checkout tag: '%s'" % git_tag)
+                return
+
+            # Rebase changes from the remote branch
             try:
                 process = self.run(['rebase', git_remote_name + '/' + git_tag])
             except exc.CommandError as e:
@@ -343,32 +336,19 @@ class GitRepo(BaseRepo):
                 return
 
         cmd = ['submodule', 'update', '--recursive', '--init']
-        cmd.extend(self.git_submodules)
         self.run(cmd, log_in_real_time=True)
 
-    def remotes(self, flat=False):
+    def remotes(self, flat=False) -> Dict:
         """Return remotes like git remote -v.
 
-        :param flat: Return a dict of ``tuple`` instead of ``dict``. Default False.
-        :type flat: bool
-
-        .. versionchanged:: 0.4.0
-
-           Has been changed from property to method
-
-        .. versionchanged:: 0.4.0
-
-           The ``flat`` argument has been added to return remotes in ``tuple`` form
-
-        .. versionchanged:: 0.4.0
-
-           This used to return a dict of tuples, it now returns a dict of dictionaries
-           with ``name``, ``fetch_url``, and ``push_url``.
+        Parameters
+        ----------
+        flat : bool
+            Return a dict of ``tuple`` instead of ``dict``, default `False`.
 
         Returns
         -------
-        dict :
-            dict of git upstream / remote URLs
+        dict of git upstream / remote URLs
         """
         remotes = {}
 
@@ -381,24 +361,7 @@ class GitRepo(BaseRepo):
             )
         return remotes
 
-    @property
-    def remotes_get(self):
-        """
-        .. versionchanged:: 0.4.0
-
-           The ``remotes_get`` property is deprecated and will be removed in 0.5. It
-           has been renamed ``remotes()`` and changed from property to a method.
-        """
-        warnings.warn(
-            "'remotes_get' is deprecated and will be removed in 0.5. "
-            "Use 'remotes()' method instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-
-        return self.remotes()
-
-    def remote(self, name, **kwargs):
+    def remote(self, name, **kwargs) -> GitRemote:
         """Get the fetch and push URL for a specified remote name.
 
         Parameters
@@ -408,24 +371,9 @@ class GitRepo(BaseRepo):
 
         Returns
         -------
-        :class:`libvcs.git.GitRemote` : 
-            Remote name and url in tuple form
-
-        .. versionchanged:: 0.4.0
-
-           The ``remote`` argument was renamed to ``name`` and will be deprecated
-           in 0.5.
+        Remote name and url in tuple form
         """
 
-        if kwargs.get('remote') is not None:
-            warnings.warn(
-                "'remote' is deprecated and will be removed in 0.5. "
-                "Use 'name' instead.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            name = kwargs.get('remote')
-
         try:
             ret = self.run(['remote', 'show', '-n', name])
             lines = ret.split('\n')
@@ -440,31 +388,16 @@ class GitRepo(BaseRepo):
         except exc.LibVCSException:
             return None
 
-    def remote_get(self, name='origin', **kwargs):
-        """
-        .. versionchanged:: 0.4.0
-
-           The ``remote_get`` method is deprecated and will be removed in 0.5.0. It has
-           been renamed ``remote`` 
-        """
-        warnings.warn(
-            "'remote_get' is deprecated and will be removed in 0.5. "
-            "Use 'remote' instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-
-        return self.remote(name=name, **kwargs)
-
     def set_remote(self, name, url, overwrite=False):
         """Set remote with name and URL like git remote add.
 
-        :param name: defines the remote name.
-        :type name: str
-        :param url: defines the remote URL
-        :type url: str
+        Parameters
+        ----------
+        name : str
+            defines the remote name.
 
-        .. versionadded:: 0.4.0
+        url : str
+            defines the remote URL
         """
 
         url = self.chomp_protocol(url)
@@ -475,35 +408,18 @@ class GitRepo(BaseRepo):
             self.run(['remote', 'add', name, url])
         return self.remote(name=name)
 
-    def remote_set(self, url, name='origin', overwrite=False, **kwargs):
-        """
-        .. versionchanged:: 0.4.0
-
-           The ``remote_set`` method is deprecated and will be removed in 0.5.0. It has
-           been renamed ``set_remote``.
-        """
-        warnings.warn(
-            "'remote_set' is deprecated and will be removed in 0.5. "
-            "Use 'set_remote' instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-
-        return self.set_remote(url=url, name=name, overwrite=overwrite, **kwargs)
-
     @staticmethod
-    def chomp_protocol(url):
+    def chomp_protocol(url) -> str:
         """Return clean VCS url from RFC-style url
 
         Parameters
         ----------
-        url : str 
+        url : str
             PIP-style url
 
         Returns
         -------
-        str :
-            URL as VCS software would accept it
+        URL as VCS software would accept it
         """
         if '+' in url:
             url = url.split('+', 1)[1]
@@ -520,13 +436,12 @@ class GitRepo(BaseRepo):
             url = url.replace('ssh://', '')
         return url
 
-    def get_git_version(self):
+    def get_git_version(self) -> str:
         """Return current version of git binary
 
         Returns
         -------
-        str :
-            git version
+        git version
         """
         VERSION_PFX = 'git version '
         version = self.run(['version'])
@@ -536,44 +451,40 @@ class GitRepo(BaseRepo):
             version = ''
         return '.'.join(version.split('.')[:3])
 
-    def status(self):
+    def status(self) -> dict:
         """Retrieve status of project in dict format.
 
         Wraps ``git status --sb --porcelain=2``. Does not include changed files, yet.
 
+        Returns
+        -------
+        Status of current checked out repository
+
         Examples
         --------
 
-        ::
-
-            print(git_repo.status())
-            {
-                "branch_oid": 'de6185fde0806e5c7754ca05676325a1ea4d6348',
-                "branch_head": 'fix-current-remote-name',
-                "branch_upstream": 'origin/fix-current-remote-name',
-                "branch_ab": '+0 -0',
-                "branch_ahead": '0',
-                "branch_behind": '0',
-            }
-
-        Returns
-        -------
-        dict :
-            Status of current checked out repository
+        >>> git_repo.status()
+        {
+            "branch_oid": 'de6185fde0806e5c7754ca05676325a1ea4d6348',
+            "branch_head": 'fix-current-remote-name',
+            "branch_upstream": 'origin/fix-current-remote-name',
+            "branch_ab": '+0 -0',
+            "branch_ahead": '0',
+            "branch_behind": '0'
+        }
         """
         return extract_status(self.run(['status', '-sb', '--porcelain=2']))
 
-    def get_current_remote_name(self):
+    def get_current_remote_name(self) -> str:
         """Retrieve name of the remote / upstream of currently checked out branch.
 
         Returns
         -------
-        str :
-            If upstream the same, returns ``branch_name``.
-            If upstream mismatches, returns ``remote_name/branch_name``.
+        If upstream the same, returns ``branch_name``.
+        If upstream mismatches, returns ``remote_name/branch_name``.
         """
         match = self.status()
 
-        if match['branch_upstream'] is None:  # no upstream set
-            return match['branch_head']
-        return match['branch_upstream'].replace('/' + match['branch_head'], '')
+        if match.branch_upstream is None:  # no upstream set
+            return match.branch_head
+        return match.branch_upstream.replace('/' + match.branch_head, '')
