@@ -1,59 +1,61 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """Subversion object for libvcs.
 
-libvcs.svn
-~~~~~~~~~~
-
 The follow are from saltstack/salt (Apache license):
 
-- :py:meth:`SubversionRepo.get_revision_file`
+- [`SubversionRepo.get_revision_file`](libvcs.svn.SubversionRepo.get_revision_file)
 
 The following are pypa/pip (MIT license):
 
-- :py:meth:`SubversionRepo.get_url_and_revision_from_pip_url`
-- :py:meth:`SubversionRepo.get_url`
-- :py:meth:`SubversionRepo.get_revision`
-- :py:meth:`~.get_rev_options`
-
-"""
-from __future__ import absolute_import, print_function, unicode_literals
-
+- [`SubversionRepo.convert_pip_url`](libvcs.svn.SubversionRepo.convert_pip_url)
+- [`SubversionRepo.get_url`](libvcs.svn.SubversionRepo.get_url)
+- [`SubversionRepo.get_revision`](libvcs.svn.SubversionRepo.get_revision)
+- [`get_rev_options`](libvcs.svn.get_rev_options)
+"""  # NOQA: E5
 import logging
 import os
 import re
+from urllib import parse as urlparse
 
-from ._compat import urlparse
-from .base import BaseRepo
+from .base import BaseRepo, VCSLocation, convert_pip_url as base_convert_pip_url
 
 logger = logging.getLogger(__name__)
 
 
+def convert_pip_url(pip_url: str) -> VCSLocation:
+    # hotfix the URL scheme after removing svn+ from svn+ssh:// re-add it
+    url, rev = base_convert_pip_url(pip_url)
+    if url.startswith('ssh://'):
+        url = 'svn+' + url
+    return VCSLocation(url=url, rev=rev)
+
+
 class SubversionRepo(BaseRepo):
     bin_name = 'svn'
     schemes = ('svn', 'svn+ssh', 'svn+http', 'svn+https', 'svn+svn')
 
-    def __init__(self, url, **kwargs):
+    def __init__(self, url, repo_dir, **kwargs):
         """A svn repository.
 
-        :param url: URL in subversion repository
-        :type url: str
+        Parameters
+        ----------
+        url : str
+            URL in subversion repository
 
-        :param svn_username: username to use for checkout and update
-        :type svn_username: str or None
+        svn_username : str, optional
+            username to use for checkout and update
 
-        :param svn_password: password to use for checkout and update
-        :type svn_password: str or None
+        svn_password : str, optional
+            password to use for checkout and update
 
-        :param svn_trust_cert: trust the Subversion server site certificate
-            (default False)
-        :type svn_trust_cert: bool
+        svn_trust_cert : bool
+            trust the Subversion server site certificate, default False
         """
         if 'svn_trust_cert' not in kwargs:
             self.svn_trust_cert = False
 
         self.rev = kwargs.get('rev')
-        BaseRepo.__init__(self, url, **kwargs)
+        BaseRepo.__init__(self, url, repo_dir, **kwargs)
 
     def _user_pw_args(self):
         args = []
@@ -63,7 +65,7 @@ class SubversionRepo(BaseRepo):
         return args
 
     def obtain(self, quiet=None):
-        self.check_destination()
+        self.ensure_dir()
 
         url, rev = self.url, self.rev
 
@@ -87,9 +89,7 @@ class SubversionRepo(BaseRepo):
         return int(dict(info_list)['Revision'])
 
     def get_revision(self, location=None):
-        """
-        Return the maximum revision for all files under a given location
-        """
+        """Return maximum revision for all files under a given location"""
 
         if not location:
             location = self.url
@@ -120,16 +120,8 @@ class SubversionRepo(BaseRepo):
             revision = max(revision, localrev)
         return revision
 
-    @classmethod
-    def get_url_and_revision_from_pip_url(cls, pip_url):
-        # hotfix the URL scheme after removing svn+ from svn+ssh:// re-add it
-        url, rev = super(SubversionRepo, cls).get_url_and_revision_from_pip_url(pip_url)
-        if url.startswith('ssh://'):
-            url = 'svn+' + url
-        return url, rev
-
     def update_repo(self, dest=None):
-        self.check_destination()
+        self.ensure_dir()
         if os.path.isdir(os.path.join(self.path, '.svn')):
             dest = self.path if not dest else dest
 
@@ -146,11 +138,7 @@ class SubversionRepo(BaseRepo):
 
 
 def get_rev_options(url, rev):
-    """Return revision options.
-
-    from pip pip.vcs.subversion.
-
-    """
+    """Return revision options. From pip pip.vcs.subversion."""
     if rev:
         rev_options = ['-r', rev]
     else:
